/*
	(C) Copyright 1997-2001 AROS - The Amiga Research OS
	$Id: menutask.c,v 1.6 2019/08/18 07:18:26 piru Exp $

	Desc:
	Lang: english

*/

#define AROS_ALMOST_COMPATIBLE 1 /* NEWLIST macro */

#include <proto/exec.h>
#include <proto/intuition.h>
#include <proto/alib.h>
#include <proto/layers.h>
#include <proto/graphics.h>
#include <proto/keymap.h>
#include <proto/cybergraphics.h>
#include <proto/dos.h>
#include <exec/memory.h>
#include <exec/alerts.h>
#include <exec/interrupts.h>
#include <exec/ports.h>
#include <intuition/intuition.h>
#include <intuition/intuitionbase.h>
#include <intuition/imageclass.h>
#include <graphics/gfxmacros.h>
#include <graphics/rpattr.h>
#include <devices/inputevent.h>
#include <devices/input.h>
#include <dos/dostags.h>
#include "inputhandler.h"
#include "intuition_intern.h"
#include "intuition_private.h"
#include "intuition_customize.h"
#include "maybe_boopsi.h"
#include "menus.h"
#include "menutask.h"
#include "smallmenu.h"
#include "smallmenu_menusupport.h"
#include "smallmenu_render.h"
#include "intuition_customizesupport.h"
#include "mosmisc.h"

#include <cybergraphx/cybergraphics.h>

#undef DEBUG
#define DEBUG 0
#include <aros/debug.h>

extern ULONG HookEntry();

#define DEBUG_CLICK(x)  ;
#define DEBUG_ADDTO(x)  ;
#define DEBUG_MAIN(x)   ;

#define CLOCKTICKS 1

/**************************************************************************************************/

/* this #defines are taken from workbench/libs/gadtools/menus.c!! */

#define TEXT_AMIGAKEY_SPACING  6

#define ITEXT_EXTRA_LEFT   2
#define ITEXT_EXTRA_RIGHT  2
#define ITEXT_EXTRA_TOP    1
#define ITEXT_EXTRA_BOTTOM 1

//static const char *subitemindicator = "";

/**************************************************************************************************/

BOOL HandleMouseMove(struct MenuHandlerData *mhd, struct SmallMenuEntry *sme,struct IntuitionBase *IntuitionBase);
void HandleMouseClick(struct InputEvent *ie, struct MenuHandlerData *mhd,
							 struct IntuitionBase *IntuitionBase);

/**************************************************************************************************/

/******************************
**  CreateMenuHandlerTask()  **
******************************/
struct Task *CreateMenuHandlerTask(APTR taskparams, struct IntuitionBase *IntuitionBase)
{
	return (struct Task *)CreateNewProcTags(NP_Name, (ULONG)MENUTASK_NAME,
											NP_CodeType, CODETYPE_PPC,
											NP_Entry, (ULONG)DefaultMenuHandler,
											NP_WindowPtr, (ULONG)-1,
											NP_PPC_Arg1, (ULONG)taskparams,
											NP_StackSize, MENUTASK_STACKSIZE,
											NP_Priority, MENUTASK_PRIORITY,
											TAG_DONE);
}

#if 0
struct Task *CreateMenuHandlerTask(APTR taskparams, struct IntuitionBase *IntuitionBase)
{
	struct Task *task;
	APTR	stack;

	task = AllocMem(sizeof (struct Task), MEMF_PUBLIC|MEMF_CLEAR);
	if (task)
	{
		stack = AllocMem(MENUTASK_STACKSIZE, MEMF_PUBLIC);
		if (stack)
		{
			NEWLIST(&task->tc_MemEntry);
			task->tc_Node.ln_Type = NT_TASK;
			task->tc_Node.ln_Name = MENUTASK_NAME;
			task->tc_Node.ln_Pri = MENUTASK_PRIORITY;

			task->tc_SPLower=stack;
			task->tc_SPUpper=(BYTE *)stack + MENUTASK_STACKSIZE;

#ifdef __MORPHOS__
			task->tc_SPReg = task->tc_SPUpper;

			{
				struct TaskInitExtension taskext;
				struct TagItem tags[4];

				taskext.Trap = TRAP_PPCTASK;
				taskext.Extension = 0;
				taskext.Tags = tags;

				tags[0].ti_Tag = TASKTAG_CODETYPE;
				tags[0].ti_Data = CODETYPE_PPC;
				tags[1].ti_Tag = TASKTAG_PC;
				tags[1].ti_Data = (ULONG)DefaultMenuHandler;
				tags[2].ti_Tag = TASKTAG_PPC_ARG1;
				tags[2].ti_Data = (ULONG)taskparams;
				tags[3].ti_Tag = TAG_END;

				if(AddTask(task, (APTR)&taskext, NULL) != NULL)
				{
					/* Everything went OK */
					return (task);
				}
			}
#else

#if AROS_STACK_GROWS_DOWNWARDS
task->tc_SPReg = (BYTE *)task->tc_SPUpper-SP_OFFSET - sizeof(APTR);
			((APTR *)task->tc_SPUpper)[-1] = taskparams;
#else
task->tc_SPReg=(BYTE *)task->tc_SPLower-SP_OFFSET + sizeof(APTR);
			*(APTR *)task->tc_SPLower = taskparams;
#endif

			if(AddTask(task, DefaultMenuHandler, NULL) != NULL)
			{
				/* Everything went OK */
				return (task);
			}
#endif
			FreeMem(stack, MENUTASK_STACKSIZE);

		} /* if(stack != NULL) */
		FreeMem(task,sizeof(struct Task));

	} /* if (task) */
	return (NULL);

}
#endif

#undef DefaultMenuHandler


/**************************************************************************************************/

/***************************
**  DefaultMenuHandler()  **
***************************/
void DefaultMenuHandler(struct MenuTaskParams *taskparams)
{
	struct IntuitionBase    *IntuitionBase = taskparams->intuitionBase;

	struct MenuHandlerData  *mhd = NULL;
	UBYTE		    *mem;
	struct MsgPort	    *port = NULL;
	ULONG  mpmask = 0;

	BOOL		    success = FALSE;

	D(bug("DefaultMenuHandler: taskparams 0x%lx\n",taskparams));

	if ((mem = AllocMem(sizeof(struct MsgPort) +
			    sizeof(struct MenuHandlerData), MEMF_PUBLIC | MEMF_CLEAR)))
	{
		port = (struct MsgPort *)mem;

		port->mp_SigBit		= AllocSignal(-1);
		if (port->mp_SigBit != (UBYTE) -1)
		{
			port->mp_Node.ln_Type	= NT_MSGPORT;
			port->mp_Flags		= PA_SIGNAL;
			port->mp_SigTask	= FindTask(NULL);
			NEWLIST(&port->mp_MsgList);

			mpmask = 1L << port->mp_SigBit;

			mhd = (struct MenuHandlerData *)(mem + sizeof(struct MsgPort));

			success = TRUE;
		}

	} /* if ((mem = AllocMem(sizeof(struct MsgPort), MEMF_PUBLIC | MEMF_CLEAR))) */

	if (success)
	{
		taskparams->MenuHandlerPort = port;
		taskparams->success = TRUE;
	}

	D(bug("DefaultMenuHandler: Signal Caller 0x%lx <%s>\n",taskparams->Caller,taskparams->Caller->tc_Node.ln_Name));

	Signal(taskparams->Caller, SIGF_INTUITION);

	if (!success)
	{
		D(bug("DefaultMenuHandler: initialization failed. waiting for parent task to kill me.\n"));

		if (mem)
		{
			FreeMem(mem, sizeof(struct MsgPort) + sizeof(struct MenuHandlerData));
		}

		Wait(0);
	}

	D(bug("DefaultMenuHandler: initialization ok. Now waiting for messages from Intuition.\n"));

	for(;;)
	{
		ULONG sigs;
		struct MenuMessage *msg;

		D(bug("DefaultMenuHandler: Wait 0x%lx\n",mpmask | timermask));

		sigs = Wait(mpmask);

		D(bug("DefaultMenuHandler: got sigmask 0x%lx (timermask %lx)\n",sigs,timermask));

		if (sigs & mpmask)
		{
			while((msg = GetMenuMessage(port, IntuitionBase)))
			{
				switch(msg->code)
				{
				case MMCODE_START:

					DEBUG_MAIN(dprintf("Defaultmenuhandler: mmcode start!\n"));

#ifdef USEWINDOWLOCK
					// let's prevent other windows from opening while menus are on
					LOCKWINDOW;
					mhd->windowlock = TRUE;
#endif

					mhd->win			= msg->win;

					if (!msg->win)
					{
						DEBUG_MAIN(dprintf("Defaultmenuhandler: no window!\n"));
						KillMenus(mhd,IntuitionBase);
						break;
					} else {
						ULONG lock;

						DEBUG_MAIN(dprintf("DefaultMenuHandler: locking ibase\n"));

						lock = LockIBase(0);

						if (!WindowValid(msg->win,IntuitionBase))
						{
							DEBUG_MAIN(dprintf("Defaultmenuhandler: bogus window!\n"));
							UnlockIBase(lock);
							KillMenus(mhd,IntuitionBase);
							break;
						}

						UnlockIBase(lock);
					}

					mhd->scr			= mhd->win->WScreen;
					mhd->dri			= (struct IntDrawInfo *)GetScreenDrawInfo(mhd->scr);

					mhd->menu			= msg->win->MenuStrip;

					/* lend menus */
					if (((struct IntWindow *)msg->win)->menulendwindow)
					{
						mhd->menu = ((struct IntWindow *)msg->win)->menulendwindow->MenuStrip;
					}

					if (!mhd->menu)
					{
						DEBUG_MAIN(dprintf("Defaultmenuhandler: no menu!\n"));
						KillMenus(mhd,IntuitionBase);
						break;
					}

					DEBUG_MAIN(dprintf("Defaultmenuhandler: menu %lx\n",mhd->menu));

					mhd->scrmousex		= mhd->scr->MouseX;
					mhd->scrmousey		= mhd->scr->MouseY;
					mhd->firstmenupick	= MENUNULL;
					mhd->keepmenuup		= TRUE;

					mhd->lastclicked = mhd->lastselected = NULL;

					mhd->backfillhook.h_Entry = (HOOKFUNC)HookEntry;
					mhd->backfillhook.h_SubEntry = (HOOKFUNC)CustomizeBackfillFunc;
					mhd->backfillhook.h_Data = &mhd->hookdata;
					mhd->hookdata.intuitionBase = IntuitionBase;
					mhd->isundermouse = FALSE;

					mhd->openseconds = msg->ie.ie_TimeStamp.tv_secs;
					mhd->openmicros = msg->ie.ie_TimeStamp.tv_micro;

					mhd->delayedopen = 0;

					if ((IBase->IControlPrefs.ic_Flags & ICF_MENUUNDERMOUSE) || ((IBase->IControlPrefs.ic_Flags & ICF_MENUMOUSEPOS) && (mhd->scr->MouseY >= mhd->scr->BarHeight) ))
						mhd->isundermouse = TRUE;

					/* close windows in the back first because
					   this is faster */
					DEBUG_MAIN(dprintf("Defaultmenuhandler: making smallmenu\n"));

					MakeMenuBarWin(mhd, IntuitionBase);

					DEBUG_MAIN(dprintf("Defaultmenuhandler: smallmenu done!\n"));

					if (mhd->rootsmk.window && mhd->entries)
					{
						HandleMouseMove(mhd, mhd->entries, IntuitionBase);

						DEBUG_MAIN(dprintf("Defaultmenuhandler: mmcode start succeed!\n"));

						mhd->active = TRUE;
						break;
					}

					DEBUG_MAIN(dprintf("Defaultmenuhandler: start failed!\n"));

					KillMenus(mhd,IntuitionBase);
					
					break;

				case MMCODE_EVENT:
					/* there might come additional messages from Intuition
					   even when we have already told it to make the menus
					   inactive, but since everything is async, this cannot
					   be avoided, so check if we are really active */

					if (mhd->active)
					{
						switch(msg->ie.ie_Class)
						{
						case IECLASS_RAWMOUSE:
							if (msg->ie.ie_Code == IECODE_NOBUTTON)
							{
								HandleMouseMove(mhd, mhd->entries, IntuitionBase);
							}
							else
							{
								HandleMouseClick(&msg->ie, mhd, IntuitionBase);
							}
							break;
						case IECLASS_RAWKEY:
							HandleRawKey(&msg->ie,mhd,IntuitionBase);
							break;

						case IECLASS_TIMER:
						case IECLASS_NEWTIMER:
							if (mhd->delayedopen)
							{
								ULONG lock = LockIBase(0);

								if (WindowsReplied(mhd->scr,IntuitionBase))
								{
									struct TagItem cmwtags[] =
									{
										{SMCREATEWINDOW_USERMP,-1},
										{SMCREATEWINDOW_POSX,CalcSubWindowXPos(mhd->delayedopen->parent,IntuitionBase)},
										{SMCREATEWINDOW_POSY,CalcSubWindowYPos(mhd->delayedopen->parent,IntuitionBase)},
										{TAG_DONE}
									};

									UnlockIBase(lock);
									CreateMenuWindow(mhd->delayedopen,mhd->scr,(struct TagItem *)&cmwtags,IntuitionBase);
									mhd->delayedopen = 0;
								} else {
									UnlockIBase(lock);
								}
							}
							break;

						}

					} /* if (mhd->active) */
					break;

				} /* switch(msg->code) */

				ReplyMenuMessage(msg, IntuitionBase);

			} /* while((msg = (struct MenuMessage *)GetMsg(port))) */
		}

		D(bug("DefaultMenuHandler: Restarting loop.\n"));

	} /* for(;;) */
}

/**************************************************************************************************/

/*******************************
**  InitDefaultMenuHandler()  **
*******************************/
BOOL InitDefaultMenuHandler(struct IntuitionBase *IntuitionBase)
{
	struct MenuTaskParams   params;
	struct Task		    *task;
	BOOL		    result = FALSE;

	D(bug("InitDefaultMenuHandler:\n"));

	params.intuitionBase = IntuitionBase;
	params.Caller	 = FindTask(NULL);
	params.success	 = FALSE;

	D(bug("InitDefaultMenuHandler: Clear SIGF_INTUITION\n"));

	SetSignal(0, SIGF_INTUITION);

	D(bug("InitDefaultMenuHandler: CreateMenuHandlerTask\n"));
	if ((task = CreateMenuHandlerTask(&params, IntuitionBase)))
	{
		D(bug("InitDefaultMenuHandler: Wait for SIGF_INTUITION\n"));

		Wait(SIGF_INTUITION);

		D(bug("InitDefaultMenuHandler: received SIGF_INTUITION\n"));

		if (params.success)
		{
			result = TRUE;
			IBase->MenuHandlerPort = params.MenuHandlerPort;
		}
		else
		{
			//RemTask(task);
		}

	} /* if ((task = CreateMenuHandlerTask(&params, IntuitionBase))) */
	else
	{
		D(bug("InitDefaultMenuHandler: CreateMenuHandlerTask failed\n"));
	}
	D(bug("InitDefaultMenuHandler: result 0x%lx\n",result));
	return result;
}

/**************************************************************************************************/

BOOL HandleMouseMove(struct MenuHandlerData *mhd,struct SmallMenuEntry *sme,struct IntuitionBase *IntuitionBase)
{
	struct SmallMenuEntry *entry = sme;
	struct SmallMenuEntry  * newactive = 0;
	struct Window *win = sme->smk->window; //this assumes menu's root window is opened!
	struct SmallMenuEntry *lastactivesub = NULL;
	BOOL onwindow = FALSE;

	for (; entry; entry = entry->next)
	{
		if (win->MouseX >= 0 && win->MouseX <= win->Width &&
			win->MouseY >= 0 && win->MouseY <= win->Height) //mouse inside THIS window
		{
			if (win->MouseY >= REALY(entry) && win->MouseY < REALY(entry) + entry->height &&
				win->MouseX >= REALX(entry) && win->MouseX < REALX(entry) + entry->width)
			{
				if (!(entry->flags & SMF_SELECTED)) //not selected yet
				{
					newactive = entry;
				}
			}
		}
		if (entry->submenu && entry->submenu->smk)
		{
			if(entry->flags & SMF_SELECTED) // selected?
			{
				lastactivesub = entry; /* remember, which menu was active */
			}
			onwindow |= HandleMouseMove(mhd,entry->submenu,IntuitionBase);
		}
	}

	if((!onwindow) && (!newactive) && (lastactivesub)) /* deselect menu in special case */
	{
		for(entry = lastactivesub->submenu; entry; entry = entry->next)
		{
			if ((entry->flags & SMF_SELECTED) && (!entry->submenu))
			{
				entry->flags &= ~SMF_SELECTED;
				RenderItem(entry->smk->window->RPort,entry,REALX(entry),REALY(entry),IntuitionBase);
			}
		}
	}

	if (!onwindow && newactive)
	for(entry = sme; entry; entry = entry->next)
	{
		if (entry->flags & SMF_SELECTED && entry != newactive)
		{
			entry->flags &= ~SMF_SELECTED;
			RenderItem(win->RPort,entry,REALX(entry),REALY(entry),IntuitionBase);
			if (entry->submenu)
			{
				CloseMenuWindow(entry->submenu,IntuitionBase);
			}
			if (entry->submenu && entry->submenu == mhd->delayedopen)
			{
				mhd->delayedopen = 0;
			}
		}
	}

	if (!onwindow && newactive)
	{
		newactive->flags |= SMF_SELECTED;
		RenderItem(win->RPort,newactive,REALX(newactive),REALY(newactive),IntuitionBase);

		if (newactive->submenu)
		{
			if (newactive->smk->flags & SMK_NEEDSDELAY)
			{
				if (newactive->submenu != mhd->delayedopen)
				{
						mhd->delayedopenseconds = IntuitionBase->Seconds;
						mhd->delayedopenmicros = IntuitionBase->Micros;
						mhd->delayedopen = newactive->submenu;
				}
			} else {
				struct TagItem cmwtags[] =
				{
					{SMCREATEWINDOW_USERMP,-1},
					{SMCREATEWINDOW_POSX,CalcSubWindowXPos(newactive,IntuitionBase)},
					{SMCREATEWINDOW_POSY,CalcSubWindowYPos(newactive,IntuitionBase)},
					{TAG_DONE}
				};

				CreateMenuWindow(newactive->submenu,win->WScreen,(struct TagItem *)&cmwtags,IntuitionBase);
			}
		} else {
			mhd->delayedopen = 0;
		}
	}

	if (newactive) mhd->lastselected = newactive;

	if (win->MouseX >= 0 && win->MouseX <= win->Width &&
		win->MouseY >= 0 && win->MouseY <= win->Height) //mouse inside THIS window
	{
		onwindow = TRUE;
	}

	return onwindow;
}

/**************************************************************************************************/

#define STICKY (IBase->IControlPrefs.ic_Flags & ICF_STICKYMENUS)
#define DELAYEDSTICKY (IBase->IControlPrefs.ic_Flags & ICF_STICKYDELAY)
#define DELAYVALID (DoubleClick(mhd->openseconds,mhd->openmicros,ie->ie_TimeStamp.tv_secs,ie->ie_TimeStamp.tv_micro))

void HandleMouseClick(struct InputEvent *ie, struct MenuHandlerData *mhd,
							 struct IntuitionBase *IntuitionBase)
{
	BOOL die = TRUE;

	switch (ie->ie_Code)
	{
		case MENUUP:
		case SELECTDOWN:
			{
				struct SmallMenuEntry *sel = FindMouseEntry(mhd->entries,IntuitionBase);

				if (sel)
				{
					if (sel->submenu || (sel->flags & SMF_NOTSELECTABLE) || !sel->parent)
					{
						if (STICKY) die = FALSE;
						if (DELAYEDSTICKY && DELAYVALID) die = TRUE;
						mhd->keepmenuup = FALSE;
						break;
					}

					DEBUG_CLICK(dprintf("HandleMouseClick: AddToSelection\n"));

					if (!((ie->ie_Code == MENUUP) && (mhd->lastclicked == mhd->lastselected) && (mhd->lastclicked != NULL)))
					{
						if (sel->flags & SMF_CHECKMARK)
						{
							HandleCheckItem(mhd,sel,IntuitionBase);
						}

						AddToSelection(mhd,sel,IntuitionBase);
					}

					mhd->lastclicked = sel;

#ifdef NOCBMPATENTS
					if ((ie->ie_Code == SELECTDOWN) && (ie->ie_Qualifier & IEQUALIFIER_RBUTTON))
					{
						die = FALSE;
					}
#endif

					break;
				}


				if (mhd->keepmenuup && STICKY) die = FALSE;
				if (DELAYEDSTICKY && DELAYVALID) die = TRUE;
				mhd->keepmenuup = FALSE;
			}
			break;

		default:
#ifdef NOCBMPATENTS
			if ((ie->ie_Code == SELECTUP) && (ie->ie_Qualifier & IEQUALIFIER_RBUTTON))
			{
				die = FALSE;
			}
			else
#endif
			{
				if (mhd->keepmenuup) die = FALSE;
				mhd->keepmenuup = FALSE;
			}
			break;
	}

	if (die)
	{
		DEBUG_CLICK(dprintf("HandleMouseClick: die!\n"));
		KillMenus(mhd,IntuitionBase);
	}
 
};

/**************************************************************************************************/

BOOL HandleCheckItem(struct MenuHandlerData *mhd, struct SmallMenuEntry *entry,
							struct IntuitionBase *IntuitionBase)
{
	/* Note: If you change something here, you probably must also change
		 menus.c/CheckMenuItemWasClicked() which is used when the
	  user uses the menu key shortcuts! */

	struct MenuItem *item = (struct MenuItem *)entry->reference;
	struct SmallMenuEntry *work;
	BOOL re_render = FALSE;
	BOOL changed = FALSE;

	if (item->Flags & MENUTOGGLE)
	{
		item->Flags ^= CHECKED;
		entry->flags ^= SMF_CHECKED;
		re_render = TRUE;
		changed = TRUE;
	}
	else
	{
		if (!(item->Flags & CHECKED))
		{
			item->Flags |= CHECKED;
			entry->flags |= SMF_CHECKED;
			re_render = TRUE;
			changed = TRUE;
		} else {
			if (!(item->MutualExclude))
			{
				item->Flags &= ~CHECKED;
				entry->flags &= ~SMF_CHECKED;
				re_render = TRUE;
				changed = TRUE;
			}
		}
	}

	if (re_render)
	{
		RenderItem(entry->smk->window->RPort,entry,REALX(entry),REALY(entry),IntuitionBase);
	}

	if (item->MutualExclude)
	{
		struct MenuItem *checkitem = 0;
		WORD		i,itemnum = entry->referencenumber;

		work = entry->parent->submenu;

		for(i = 0; (i < 32) && work; i++, work = work->next)
		{
			checkitem = work->reference;
			if ((i != itemnum) && (item->MutualExclude & (1L << i)) &&
				((checkitem->Flags & (CHECKED | CHECKIT)) == (CHECKIT | CHECKED)))
			{
				checkitem->Flags &= ~CHECKED;
				work->flags &= ~SMF_CHECKED;
				RenderItem(work->smk->window->RPort,work,REALX(work),REALY(work),IntuitionBase);
			}
		}

	} /* if (item->MutualExclude) */

	return changed;
}

/**************************************************************************************************/
 
/**************************************************************************************************/
void AddToSelection(struct MenuHandlerData *mhd, struct SmallMenuEntry *entry,struct IntuitionBase *IntuitionBase)
{
	ULONG menunum=-1,menuitemnum=-1,submenunum=-1;
	struct SmallMenuEntry *work;

	DEBUG_ADDTO(dprintf("AddToSelection: called\n"));

	if (entry)
	{
		//check if we're not disabled (checking whole family isn't really necessary, but oh well... ;)
		for (work = entry; work; work = work->parent)
		{
			if (work->flags & SMF_DISABLED) return;
		}
	}

	DEBUG_ADDTO(dprintf("AddToSelection: disable passed ok\n"));

	/*
	Now we need to get the FULLMENUNUM. god, this cbm-crap-menu-API sucks so much!!!
	too bad this doesn't allow unlimited menu depth :(
	*/

	if (entry->parent->flags & SMF_MENUSTRIP)
	{
		menuitemnum = entry->referencenumber;
		menunum = entry->parent->referencenumber;
	} else {
		submenunum = entry->referencenumber;
		menuitemnum = entry->parent->referencenumber;
		if (entry->parent->parent) //null should never happen here anyway, but.. ;)
		{
			menunum = entry->parent->parent->referencenumber;
		}
	}

	DEBUG_ADDTO(dprintf("AddToSelection: menu %ld menuitem %ld submenuitem %ld\n",menunum,menuitemnum,submenunum));

	if ((menunum != -1) && (menuitemnum != -1))
	{
		struct MenuItem *item = NULL;
		UWORD men = FULLMENUNUM(menunum, menuitemnum, submenunum);

		item = (struct MenuItem *)entry->reference;

		if (item && (ItemAddress(mhd->menu, men) == item))
		{
			UWORD men = FULLMENUNUM(menunum, menuitemnum, submenunum);

			DEBUG_ADDTO(dprintf("AddToSelection: adding item %lx\n",item));

			if (mhd->firstmenupick == MENUNULL)
			{
				mhd->firstmenupick = men;
			}
			else if (men != mhd->lastmenupick)
			{
				struct MenuItem *checkitem, *prevcheckitem = NULL;
				UWORD checkmen = mhd->firstmenupick;

				/* Remove men from pick queue, if it was already in there
				   and then add it at the end of the pick queue */

				while(checkmen != MENUNULL)
				{
					checkitem = ItemAddress(mhd->menu, checkmen);

					if (checkmen == men)
					{
						if (prevcheckitem == NULL)
						{
							mhd->firstmenupick = checkitem->NextSelect;
						}
						else
						{
							prevcheckitem->NextSelect = checkitem->NextSelect;
						}
					}

					checkmen = checkitem->NextSelect;
					prevcheckitem = checkitem;

				} /* while(checkmen != MENUNULL) */

				checkitem->NextSelect = men;

			} /* else if (men != mhd->lastmenupick) */

			mhd->lastmenupick = men;
			item->NextSelect = MENUNULL;

		} /* if (item) */

	} /* if ((menunum != -1) && (menuitemnum != -1)) */
}


/**************************************************************************************************/

